library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity GDG1500 is
   port(
      A            : in  unsigned(15 downto 0);	-- vstupy adresa
--      NP		       : in  std_logic;
		nRESET       : in  std_logic;
		nWR          : in  std_logic;
		nRD          : in  std_logic;
		nIORQ        : in  std_logic;
		nMREQ        : in  std_logic;
		nRFSH        : in  std_logic;
--      GATE         : in  std_logic;	-- GND
--      CL           : in  std_logic;	-- GND
--      FI           : in  std_logic;	-- Z80 hodiny vstup, nepouzito, odvozeno od FI0
      CLKIN        : in  std_logic;		-- hodiny 14.318M

		nCLK         : out  std_logic;	--*
		nSYN         : out  std_logic;	--*
		S157         : out  std_logic;	-- pristup CPU do VRAM
		P            : out  unsigned(10 downto 0);	-- adresa CGRAM
		nLOAD        : out  std_logic;	--*
		ABC          : out  std_logic;	-- nepouzito
		HSY          : out  std_logic;	--*
		BLNK         : out  std_logic;	--*
		Q3           : out  std_logic;	-- adresa mikroradku
		Q2           : out  std_logic;	-- adresa mikroradku
		Q1           : out  std_logic;	-- adresa mikroradku
		PRC          : out  std_logic;	-- port tiskarny (nepouzito pro MZ1500)
		COLR         : out  std_logic;	-- pixel clock
		nWA          : out  std_logic;	-- wait pri pristupu do VRAM (a ROM ?!?)
		nHBLK        : out  std_logic;	--*
		nVBLK        : out  std_logic;	--*
		SEL          : out  std_logic;	-- CAS pro DRAM
		FI0          : out  std_logic;	-- vystup negovanych hodin pro CPU
		nRAS         : out  std_logic;	-- kopiruje nMREQ
      nCS0         : out  std_logic;	-- IPL ROM 0000-0FFF
		nCSE			 : out  std_logic 	-- VRAM I/O E000-E00F
);
end GDG1500;

architecture bhv of GDG1500 is

----------------------------------------------------------------
-- signal definitions
----------------------------------------------------------------

signal INH1 : std_logic; 	-- 0000-0FFF ROM
signal INH2 : std_logic;   -- D000-FFFF VRAM
signal INH3 : std_logic;   -- D000-FFFF n.c.
signal Z80CLK : std_logic; 
signal interBLNK : std_logic; 
signal sync_nMREQ : std_logic; 
signal sync2_nMREQ : std_logic; 
signal sync3_nMREQ : std_logic; 
signal sync4_nMREQ : std_logic; 
signal interS157 : std_logic; 
signal inter_nWA : std_logic; 
signal inter_nWAIT0 : std_logic; 
signal inter_nHBLK : std_logic; 
signal inter_nVBLK : std_logic; 
signal inter_nLOAD : std_logic; 
signal inter_nCLK : std_logic; 
signal inter_nCS0 : std_logic; 
signal inter_SEL : std_logic; 
signal DRAM : std_logic; 

signal HCLK : unsigned(9 downto 0);
signal VCLK : unsigned(8 downto 0);
signal VRAMCOUNTER : unsigned(10 downto 0);

begin

VRAMCOUNTER(10) <= '0';
ABC <= '0';
--nCSE <= '0' when (A(15 downto 4) = "111000000000") else '1';
nCSE <= '0' when ((A(15 downto 4) = "111000000000") and (INH2 = '1') and (INH3 = '1')) else '1';
inter_nCS0 <= '0' when (INH1 = '1') and (nRD = '0') and (nMREQ = '0') and (A(15 downto 12) = "0000") else '1';
nCS0 <= inter_nCS0;
PRC <= '0' when (nIORQ = '0') and (A(7 downto 1) = "1111111") else '1';
S157 <= interS157;
BLNK <= interBLNK;
nWA <= inter_nWA and inter_nWAIT0;
P(10 downto 0) <= A(10 downto 0) when interS157 = '0' else VRAMCOUNTER(10 downto 0);
nHBLK <= inter_nHBLK;
nVBLK <= inter_nVBLK;
nLOAD <= inter_nLOAD;
DRAM <= '1' when ((A(15 downto 12) = 0) and (INH1 = '0')) or ((A(15 downto 12) > 0) and (A(15 downto 12) < 13)) or ((A(15 downto 12) > 12) and (INH2 = '0') and (INH3 = '1')) else '0';
inter_SEL <= '1' when (sync3_nMREQ = '0') and (nMREQ = '0') and (DRAM = '1') and (nRFSH = '1') else '0';
SEL <= inter_SEL;
nRAS <= nMREQ;

process(CLKIN)
begin
  if (falling_edge(CLKIN)) then
    if (HCLK = 911) then
	   HCLK <= "0000000000";
	 else
	   HCLK <= HCLK + 1;
	 end if;
	 if (HCLK = 0) then
	   inter_nHBLK <= '1';
	 end if;
	 if (HCLK = 640) then
	   inter_nHBLK <= '0';
		interBLNK <= '1';
	 end if;
	 if (HCLK = 645) then
		 if (VCLK = 261) then
			VCLK <= "000000000";
		 else
			VCLK <= VCLK + 1;
		 end if;
		 if (VCLK = 0) then
		   inter_nVBLK <= '1';
--			VRAMCOUNTER(10 downto 0) <= "00000000000";
		 end if;
		 if (VCLK = 200) then
		   inter_nVBLK <= '0';
		 end if;
--		 if (VCLK(2 downto 0) = "111") then
--		 else
--		   VRAMCOUNTER <= VRAMCOUNTER - 40;
--		 end if;
--		 VRAMCOUNTER(9 downto 0) <= TO_UNSIGNED(40 * VCLK(7 downto 3),10);
		 VRAMCOUNTER(9 downto 0) <= (VCLK(7 downto 3) & "00000") + ("00" & VCLK(7 downto 3) & "000");
		 Q1 <= VCLK(0);
		 Q2 <= VCLK(1);
		 Q3 <= VCLK(2);
	 end if;
	 if (HCLK = 657) then
	   if (VCLK = 221) then
		  nSYN <= '0';
		end if;
	   if (VCLK = 224) then
		  nSYN <= '1';
		end if;
	 end if;
	 if (HCLK = 720) then
	   HSY <= '0';
	 end if;
	 if (HCLK = 785) then
	   HSY <= '1';
	 end if;
	 if (HCLK = 881) then
		interBLNK <= '0';
	 end if;
	 if (HCLK(3 downto 0) = "0001") then
	   inter_nLOAD <= '0';
	 end if;
	 if (HCLK(3 downto 0) = "0010") then
	   inter_nLOAD <= '1';
		if (inter_nVBLK = '0') then
--		  VRAMCOUNTER(10 downto 0) <= "00000000000";
		elsif (inter_nHBLK = '1') then
		  VRAMCOUNTER(9 downto 0) <= VRAMCOUNTER(9 downto 0) + 1;
		end if;
	 end if;
	 if (HCLK(1 downto 0) = "01") then
--	   FI0 <= '0';
--	   Z80CLK <= '1';
		sync_nMREQ <= nMREQ;
	 end if;
	 if (HCLK(1 downto 0) = "11") then
--	   FI0 <= '1';
--	   Z80CLK <= '0';
--		sync2_nMREQ <= sync15_nMREQ;
	 end if;
	 COLR <= not ( HCLK(1) xor HCLK(0) );
	 FI0 <= not ( HCLK(1) xor HCLK(0) );
    Z80CLK <= HCLK(1) xor HCLK(0);
	 inter_nCLK <= not HCLK(0);
	 sync4_nMREQ <= sync3_nMREQ;
	 sync3_nMREQ <= sync2_nMREQ;
	 sync2_nMREQ <= sync_nMREQ;
  end if;
end process;

process(nRESET,A,nWR,nIORQ,Z80CLK)
begin
  if (nRESET = '0') then 
	 INH1 <= '1';
	 INH2 <= '1';
	 INH3 <= '1';
  elsif (rising_edge(Z80CLK)) then
    if (nIORQ = '0') and (nWR = '0') and (A(7 downto 3) = "11100") then
	   if (A(2 downto 0) = 0) then
		  INH1 <= '0';
		elsif (A(2 downto 0) = 1) then
		  INH2 <= '0';
		elsif (A(2 downto 0) = 2) then
		  INH1 <= '1';
		elsif (A(2 downto 0) = 3) then
		  INH2 <= '1';
		elsif (A(2 downto 0) = 4) then
		  INH1 <= '1';
		  INH2 <= '1';
		  INH3 <= '1';
		elsif (A(2 downto 0) = 5) then
		  INH3 <= '0';
		elsif (A(2 downto 0) = 6) then
		  INH3 <= '1';
		end if;
	 end if;
  end if;
end process;

process(nMREQ,A,interBLNK,nRESET)
begin
  if (falling_edge(nMREQ)) then
    if (interBLNK = '0') and (A(15 downto 12) = "1101") then	-- Dxxxh
	   inter_nWA <= '0';
	 end if;
  end if;
  if (interBLNK = '1') or (nRESET = '0') then
    inter_nWA <= '1';
  end if;
end process;

process(sync_nMREQ,nMREQ,A,Z80CLK,inter_nWA,nRESET)
begin
  if (falling_edge(Z80CLK)) and (sync_nMREQ = '0') and (inter_nWA = '1') and (A(15 downto 12) = "1101") and (INH2 = '1') and (INH3 = '1') then
	 interS157 <= '0';
  end if;
  if (nMREQ = '1') or (nRESET = '0') then
	 interS157 <= '1';
  end if;
end process;

process(Z80CLK,inter_nCS0,nRESET,inter_nWAIT0)
begin
  if (nRESET = '0') then
    inter_nWAIT0 <= '1';
  elsif (rising_edge(Z80CLK)) then
    if (inter_nWAIT0 = '0') then
		inter_nWAIT0 <= '1';
	 elsif ((inter_nCS0 = '0') and (sync3_nMREQ = '1')) then
		inter_nWAIT0 <= '0';
	 end if;
  end if;
end process;

--process(CLKIN,inter_nCLK)
----process(CLKIN,interS157)
----process(CLKIN,inter_nLOAD)
--begin
--  if rising_edge(CLKIN) then
----	S157 <= interS157;
----	nLOAD <= inter_nLOAD;
----	nCLK <= not CLKIN;
--	nCLK <= inter_nCLK;
	nCLK <= not inter_nCLK;
--  end if;
--end process;

end bhv; 
